

#include <stdarg.h>

void pr(char *fmt, ... ){
        char buf[128]; // resulting string limited to 128 chars
        va_list args;
        va_start (args, fmt );
        vsnprintf(buf, 128, fmt, args);
        va_end (args);
        Serial.println(buf);

}

// Inputs
// AIN0 (pin 6) - potentiometer
// Analogue 0 - HC-SR04 echo pin
// Analogue 1 - Envelope detector
// Analogue 2 - Final amplifier output

// Outputs
// Digital 4 - HC-SR04 trigger pin
// Digital 5 - a pin to twiddle to show what is going on
// Digital 9 - charge the capacitor in the envelope detector

 
#define TRIGGER_PIN 4
#define DEBUG_PIN 5
#define AIN0 6
#define AIN1 7
#define CHARGE_PIN 9



volatile unsigned t1ovfcnt = 0;
volatile char oneshot=0;

#define BUFFER_SIZE 96

volatile unsigned long int buffer[BUFFER_SIZE];
volatile byte head = 0;
volatile byte tail = 0;

unsigned long basetime;
unsigned long echotime;
unsigned long phasetime;

 
void setup() 
{
 Serial.begin(9600);
 while (!Serial) { ; /* wait for serial port to connect. Needed for native USB */}


 TCCR1A = 0x0;    // put timer1 in normal mode
 TCCR1B = 0x2;    // change prescaler to divide clock by 8

 // clear any pending capture or overflow interrupts
 TIFR1 = (1<<ICF1) | (1<<TOV1);
 // Enable input capture and overflow interrupts
 TIMSK1 |= (1<<ICIE1) | (1<<TOIE1);
 
 pinMode(TRIGGER_PIN, OUTPUT);
 pinMode(DEBUG_PIN, OUTPUT);
 pinMode(CHARGE_PIN, OUTPUT);
 pinMode(AIN0, INPUT);
 pinMode(AIN1, INPUT);
 DIDR1  |=  (1<< AIN0D);         // disable digital input buffer AIN0/1
 DIDR1  |=  (1<< AIN1D);

 ADCSRA=0; // ADC Control and Status Register A
 ADCSRB=(1<<ACME); // enable muliplxer
 ADMUX=0; // ADC Multiplexer Selection Register
 ACSR=(1<<ACIC); // Analog Comparator Input Capture by Timer/Counter 1: On
}


void loop() 
{
 byte newhead;
 unsigned long time1;

 digitalWrite(CHARGE_PIN, 1); 
 delay(50);
 digitalWrite(CHARGE_PIN, 0); 

 
 time1=millis();
 
 digitalWrite(TRIGGER_PIN, 0);
 delayMicroseconds(4); 
 digitalWrite(TRIGGER_PIN, 1);   
 delayMicroseconds(10); 
 digitalWrite(TRIGGER_PIN, 0);   


 TCCR1B &= ~(1 << ICES1);   //switch edge detector to detect the fall next time
 ADMUX=0; // HC-SR04 echo pin to analogue comparator
 delayMicroseconds(10);

 head=tail;
 while(1)
 {
  if(head!=tail)
  {
   newhead = (head + 1) % BUFFER_SIZE;
   basetime=buffer[newhead];
   break;
  }
  if((millis()-time1)>50) break;
 }

 delayMicroseconds(5);
 
 TCCR1B |= (1<< ICES1);    //switch edge detector to detect the rise
 ADMUX=1;  // Envelope detector to analogue comparator
 delayMicroseconds(10);
 
 head=tail;
 while(1)
 {
  if(head!= tail) 
  {
   digitalWrite(DEBUG_PIN, 1);
   newhead = (head + 1) % BUFFER_SIZE;
   echotime=buffer[newhead];
   break;
  }
  if((millis()-time1)>50) break;
 }

 delayMicroseconds(50);
 oneshot=1;
 ADMUX=2;  // Final amplifier output to analogue comparator
 
 head=tail;
 while(1)
 {
  if(head!= tail) 
  {
   newhead = (head + 1) % BUFFER_SIZE;
   phasetime=buffer[newhead];
   break;
  }
  if((millis()-time1)>50) break;
 }

 ADMUX=0; 
 oneshot=0;

 digitalWrite(DEBUG_PIN, 0);
 pr("nd %lu %lu",(echotime-basetime)/(unsigned long)2,((phasetime-basetime)/(unsigned long)2) % (unsigned long)25);

 delay(100);
}


ISR(TIMER1_OVF_vect) 
{ 
 t1ovfcnt++;              // keep track of overflows
}


ISR(TIMER1_CAPT_vect) 
{
 unsigned long t1temp;
 if(oneshot) ADMUX=0;
 tail=(tail+1) % BUFFER_SIZE;
 t1temp = ICR1;
 buffer[tail] = ((unsigned long)t1ovfcnt << 16) | t1temp;
}






